
import datetime
import os
import urllib
from gettext import gettext as _

import gtk

from gimmie_base import ItemSource
from gimmie_file import FileItem
from gimmie_util import recent_model


class RecentlyUsed(ItemSource):
    '''
    Recently-used documents, log stored in ~/.recently-used.
    '''
    def __init__(self, name, icon = "stock_calendar"):
        ItemSource.__init__(self, name=name, icon=icon)

        # NOTE: Account for differences in signal arity in the two recent backends
        recent_model.connect("changed", lambda m, list=None: self.emit("reload"))

    def get_items_uncached(self):
        for item in recent_model.get_list():
            # skip private items (like porn)
            if item.get_private():
                continue

            # Check whether to include this item
            if self.include_recent_item(item):
                yield FileItem(item.get_uri(), item.get_timestamp())

    def include_recent_item(self, item):
        return True


class RecentlyUsedOfMimeType(RecentlyUsed):
    '''
    Recently-used items filtered by a set of mimetypes.
    '''
    def __init__(self, name, icon, mimetype_list):
        RecentlyUsed.__init__(self, name, icon)
        self.mimetype_list = mimetype_list

    def include_recent_item(self, item):
        item_mime = item.get_mime_type()
        for mimetype in self.mimetype_list:
            if hasattr(mimetype, "match") and mimetype.match(item_mime) \
                   or item_mime == mimetype:
                return True
        return False


class RecentAggregate(ItemSource):
    '''
    This ItemSource subclass aggregates all the items from a list of
    ItemSources, by including the first Item encountered of a URI and
    filtering duplicates.
    '''
    def __init__(self, sources):
        ItemSource.__init__(self, name=_("Recently Used"), icon="stock_calendar")

        # Sources provide the real items we will display
        self.sources = sources
        for source in self.sources:
            self._listen_to_source(source)

    def _listen_to_source(self, source):
        source.connect("reload", lambda x: self.emit("reload"))

    def get_items_uncached(self):
        item_uris = {}

        # Find items matching recent uris
        for source in self.sources:
            for item in source.get_items_uncached():
                uri = item.get_uri()
                # Block special items the source might include like "Create
                # New Note" and avoid duplicate items for the same URI.
                if not item.is_special() and uri and uri not in item_uris:
                    item_uris[uri] = item
                    yield item
